package com.kalyshev.yan.computer.controller;

import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.configuration.OpenAPI30Configuration;
import com.kalyshev.yan.monitor.controller.MonitorDto;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(OpenAPI30Configuration.API_PREFIX + "/computer")
public class ComputerController {
    private final ComputerService computerService;
    public ComputerController(ComputerService computerService) {
        this.computerService = computerService;
    }
    @GetMapping("/{id}")
    public ComputerDto getStudent(@PathVariable Long id) {
        return new ComputerDto(computerService.findComputer(id));
    }
    @GetMapping("/")
    public List<ComputerDto> getComputers() {
        return computerService.findAllComputers().stream()
                .map(ComputerDto::new)
                .toList();
    }
    @GetMapping("/filter")
    public List<ComputerDto> getFilteredComputers(@RequestParam(value = "id", required = false) Long id,
                                                 @RequestParam(value = "modelName", required = false) String modelName,
                                                 @RequestParam(value = "serialNum", required = false) String serialNum,
                                                 @RequestParam(value = "monitorId", required = false) Long monitorId,
                                                 @RequestParam(value = "cabinetId", required = false) Long cabinetId) {
        return computerService.findFilteredComputers(id, modelName, serialNum, monitorId, cabinetId).stream()
                .map(ComputerDto::new)
                .toList();
    }
    @PostMapping("/")
    public ComputerDto createComputer(@RequestBody @Valid ComputerDto computerDto) {
        return new ComputerDto(computerService.addComputer(computerDto.getModelName(), computerDto.getSerialNum(), computerDto.getMonitorId()));
    }
    @PostMapping("/{id}/monitor")
    public MonitorDto setMonitorComputer(@PathVariable Long id,
                                         @RequestParam("monitorId") Long monitorId) {
        return new MonitorDto(computerService.setMonitor(monitorId, id));
    }
    @PutMapping("/{id}")
    public ComputerDto updateComputer(@PathVariable Long id,
                                   @RequestBody @Valid ComputerDto computerDto) {
        return new ComputerDto(computerService.updateComputer(id, computerDto.getModelName(), computerDto.getSerialNum(), computerDto.getMonitorId(), computerDto.getCabinetId()));
    }
    @DeleteMapping("/{id}")
    public ComputerDto deleteComputer(@PathVariable Long id) {
        return new ComputerDto(computerService.deleteComputer(id));
    }
}