package com.kalyshev.yan.monitor.controller;

import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.configuration.OpenAPI30Configuration;
import com.kalyshev.yan.monitor.service.MonitorService;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@RestController
@RequestMapping(OpenAPI30Configuration.API_PREFIX  + "/monitor")
public class MonitorController {
    private final MonitorService monitorService;
    private final ComputerService computerService;
    public MonitorController(MonitorService monitorService,
                             ComputerService computerService) {
        this.monitorService = monitorService;
        this.computerService = computerService;
    }
    @GetMapping("/{id}")
    public MonitorDto getMonitor(@PathVariable Long id) {
        return new MonitorDto(monitorService.findMonitor(id));
    }
    @GetMapping("/")
    public List<MonitorDto> getMonitors() {
        return monitorService.findAllMonitors().stream()
                .map(MonitorDto::new)
                .toList();
    }
    @GetMapping("/filter")
    public List<MonitorDto> getFilteredmonitors(@RequestParam(value = "id", required = false) Long id,
                                                 @RequestParam(value = "modelName", required = false) String modelName) {
        return monitorService.findFilteredMonitors(id, modelName).stream()
                .map(MonitorDto::new)
                .toList();
    }
    @PostMapping("/")
    public MonitorDto createMonitor(@RequestBody @Valid MonitorDto monitorDto) {
        return new MonitorDto(monitorService.addMonitor(monitorDto.getModelName()));
    }
    @PutMapping("/{id}")
    public MonitorDto updateMonitor(@PathVariable Long id,
                                 @RequestBody @Valid MonitorDto monitorDto) {
        return new MonitorDto(monitorService.updateMonitor(id, monitorDto.getModelName()));
    }
    @DeleteMapping("/{id}")
    public MonitorDto deleteMonitor(@PathVariable Long id) {
        return new MonitorDto(computerService.deleteMonitorWithRelation(id));
    }
}