package com.kalyshev.yan.monitor.service;

import com.kalyshev.yan.monitor.model.Monitor;
import com.kalyshev.yan.monitor.repository.MonitorNotFoundException;
import com.kalyshev.yan.monitor.repository.MonitorRepository;
import com.kalyshev.yan.util.validation.ValidatorUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Optional;

@Service
public class MonitorService {
    private final MonitorRepository monitorRepository;
    private final ValidatorUtil validatorUtil;
    public MonitorService(MonitorRepository monitorRepository,
                          ValidatorUtil validatorUtil) {
        this.monitorRepository = monitorRepository;
        this.validatorUtil = validatorUtil;
    }
    @Transactional
    public Monitor addMonitor(String modelName) {
        if (!StringUtils.hasText(modelName)) {
            throw new IllegalArgumentException("Monitor model name is null or empty");
        }
        final Monitor monitor = new Monitor(modelName);
        validatorUtil.validate(monitor);
        return monitorRepository.save(monitor);
    }
    @Transactional(readOnly = true)
    public Monitor findMonitor(Long id) {
        final Optional<Monitor> monitor = monitorRepository.findById(id);
        return monitor.orElseThrow(() -> new MonitorNotFoundException(id));
    }
    @Transactional(readOnly = true)
    public List<Monitor> findAllMonitors() {
        return monitorRepository.findAll();
    }
    @Transactional(readOnly = true)
    public List<Monitor> findFilteredMonitors(Long id, String modelName) {
        return monitorRepository.findFilteredMonitors(id, modelName);
    }
    @Transactional
    public Monitor updateMonitor(Long id, String modelName) {
        if (!StringUtils.hasText(modelName)) {
            throw new IllegalArgumentException("Monitor model name is null or empty");
        }
        final Monitor currentMonitor = findMonitor(id);
        currentMonitor.setModelName(modelName);
        validatorUtil.validate(currentMonitor);
        return monitorRepository.save(currentMonitor);
    }
    @Transactional
    public void deleteMonitor(Monitor monitor) {
        monitorRepository.delete(monitor);
    }
    @Transactional
    public void deleteAllMonitors() { monitorRepository.deleteAll(); }
}