package com.kalyshev.yan;

import com.kalyshev.yan.cabinet.model.Cabinet;
import com.kalyshev.yan.cabinet.service.CabinetService;
import com.kalyshev.yan.computer.model.Computer;
import com.kalyshev.yan.computer.repository.ComputerNotFoundException;
import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.monitor.model.Monitor;
import com.kalyshev.yan.monitor.service.MonitorService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;

@SpringBootTest
public class JpaComputerTests {
    private static final Logger log = LoggerFactory.getLogger(JpaComputerTests.class);
    @Autowired
    private ComputerService computerService;
    @Autowired
    private MonitorService monitorService;
    @Autowired
    private CabinetService cabinetService;

    @Test
    void testComputerPartialCreate() {
        computerService.deleteAllComputers();
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Computer computer = computerService.addComputer("Computer", "w7894572", monitor.getId());
        log.info(computer.toString());
        Assertions.assertNotNull(computer.getId());
        Assertions.assertNull(computer.getCabinet());
    }
    @Test
    void testComputerFullCreate() {
        computerService.deleteAllComputers();
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Cabinet cabinet = cabinetService.addCabinet("18a");
        final Computer computer = computerService.addComputer("Computer", "w7894572", monitor.getId());
        computer.setCabinet(cabinet);
        log.info(computer.toString());
        Assertions.assertNotNull(computer.getId());
        Assertions.assertEquals(cabinet, computer.getCabinet());
    }
    @Test
    void testComputerDelete() {
        computerService.deleteAllComputers();
        final Monitor monitor = monitorService.addMonitor("Asus");
        final Computer computer = computerService.addComputer("Computer", "w7894572", null);
        log.info(computer.toString());
        computerService.deleteComputer(computer.getId());
        Assertions.assertThrows(ComputerNotFoundException.class, () -> computerService.findComputer(computer.getId()));
    }
    @Test
    void testComputerRead() {
        computerService.deleteAllComputers();
        final Computer computer = computerService.addComputer("Computer", "w7894572", null);
        log.info(computer.toString());
        final Computer findComputer = computerService.findComputer(computer.getId());
        log.info(findComputer.toString());
        Assertions.assertEquals(computer, findComputer);
    }
    @Test
    void testComputerReadNotFound() {
        computerService.deleteAllComputers();
        Assertions.assertThrows(ComputerNotFoundException.class, () -> computerService.findComputer(-1L));
    }
    @Test
    void testComputerReadAll() {
        computerService.deleteAllComputers();
        final Monitor monitor = monitorService.addMonitor("Asus");
        computerService.addComputer("Computer", "w7894572", null);
        computerService.addComputer("Another comp", "3453s", null);
        final List<Computer> computers = computerService.findAllComputers();
        log.info(computers.toString());
        Assertions.assertEquals(computers.size(), 2);
    }
    @Test
    void testComputerReadAllEmpty() {
        computerService.deleteAllComputers();
        final List<Computer> computers = computerService.findAllComputers();
        log.info(computers.toString());
        Assertions.assertEquals(computers.size(), 0);
    }
}
