package com.kalyshev.yan;

import com.kalyshev.yan.computer.model.Computer;
import com.kalyshev.yan.computer.service.ComputerService;
import com.kalyshev.yan.monitor.model.Monitor;
import com.kalyshev.yan.monitor.repository.MonitorNotFoundException;
import com.kalyshev.yan.monitor.service.MonitorService;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.List;
@SpringBootTest
public class JpaMonitorTests {
    private static final Logger log = LoggerFactory.getLogger(JpaMonitorTests.class);
    @Autowired
    private MonitorService monitorService;
    @Autowired
    private ComputerService computerService;
    @Test
    void testMonitorCreate() {
        computerService.deleteAllMonitorsWithRelations();
        final Monitor monitor = monitorService.addMonitor("Asus");
        log.info(monitor.toString());
        Assertions.assertNotNull(monitor.getId());
    }
    @Test
    void testMonitorRead() {
        computerService.deleteAllMonitorsWithRelations();
        final Monitor monitor = monitorService.addMonitor("Asus");
        log.info(monitor.toString());
        final Monitor findMonitor = monitorService.findMonitor(monitor.getId());
        log.info(findMonitor.toString());
        Assertions.assertEquals(monitor, findMonitor);
    }
    @Test
    void testMonitorDelete() {
        computerService.deleteAllMonitorsWithRelations();
        final Monitor monitor = monitorService.addMonitor("Asus");
        log.info(monitor.toString());
        computerService.deleteMonitorWithRelation(monitor.getId());
        Assertions.assertThrows(MonitorNotFoundException.class, () -> monitorService.findMonitor(monitor.getId()));
    }
    @Test
    void testMonitorReadNotFound() {
        computerService.deleteAllMonitorsWithRelations();
        Assertions.assertThrows(MonitorNotFoundException.class, () -> monitorService.findMonitor(-1L));
    }
    @Test
    void testMonitorReadAll() {
        computerService.deleteAllMonitorsWithRelations();
        monitorService.addMonitor("Asus");
        monitorService.addMonitor("HP");
        final List<Monitor> monitors = monitorService.findAllMonitors();
        log.info(monitors.toString());
        Assertions.assertEquals(monitors.size(), 2);
    }
    @Test
    void testMonitorReadAllEmpty() {
        computerService.deleteAllMonitorsWithRelations();
        final List<Monitor> monitors = monitorService.findAllMonitors();
        log.info(monitors.toString());
        Assertions.assertEquals(monitors.size(), 0);
    }
    @Test
    void testMonitorGetComputer() {
        computerService.deleteAllMonitorsWithRelations();
        computerService.deleteAllComputers();
        Monitor monitor = monitorService.addMonitor("Asus");
        Computer computer = computerService.addComputer("Model", "6sfv4", monitor.getId());
        log.info(computer.toString());
        Computer fetchedComputer = computerService.findComputerByMonitor(monitor);
        Assertions.assertEquals(computer, fetchedComputer);
    }
}
